<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Http;

class MainController extends Controller
{
    // Function to calculate distance using the Haversine formula
    private function haversineGreatCircleDistance($latitudeFrom, $longitudeFrom, $latitudeTo, $longitudeTo, $earthRadius = 6371) {
        $latFrom = deg2rad($latitudeFrom);
        $lonFrom = deg2rad($longitudeFrom);
        $latTo = deg2rad($latitudeTo);
        $lonTo = deg2rad($longitudeTo);

        $latDelta = $latTo - $latFrom;
        $lonDelta = $lonTo - $lonFrom;

        $angle = 2 * asin(sqrt(pow(sin($latDelta / 2), 2) +
            cos($latFrom) * cos($latTo) * pow(sin($lonDelta / 2), 2)));
        return $angle * $earthRadius; // Distance in km
    }

    // Function to geocode address into lat/lon
    private function geocodeAddress($address) {
        $apiKey = "32d9575c8f1b4a919c194218f09b987f"; // Use environment variable for the API key
        $address = urlencode($address);
        $url = "https://api.opencagedata.com/geocode/v1/json?q={$address}&key={$apiKey}&limit=1";

        $response = Http::get($url);
        $json = $response->json();

        if (!empty($json['results'])) {
            $latitude = $json['results'][0]['geometry']['lat'];
            $longitude = $json['results'][0]['geometry']['lng'];
            return ['lat' => $latitude, 'lng' => $longitude];
        } else {
            return false; // Geocoding failed
        }
    }

    // Function to reverse geocode lat/lon into an address (excluding street-level details)
    private function reverseGeocode($latitude, $longitude) {
        $apiKey = "32d9575c8f1b4a919c194218f09b987f"; // Use environment variable for the API key
        $url = "https://api.opencagedata.com/geocode/v1/json?q={$latitude},{$longitude}&key={$apiKey}&limit=1";

        $response = Http::get($url);
        $json = $response->json();

        if (!empty($json['results'])) {
            $components = $json['results'][0]['components'];
            $city = $components['city'] ?? $components['town'] ?? $components['village'] ?? '';
            $state = $components['state'] ?? '';
            $country = $components['country'] ?? '';
            return trim("$city, $state, $country");
        } else {
            return false; // Reverse geocoding failed
        }
    }

    // Method to get nearby agencies
    public function getNearbyAgencies(Request $request) {
        $userLat = $request->input('lat');
        $userLng = $request->input('lng');
        $user_address = $this->reverseGeocode($userLat, $userLng);

        // Fetch agencies from database
        $agencies = DB::table('agencies')->select('id', 'name', 'address', 'phone', 'latitude', 'longitude')->get();

        $nearbyAgencies = [];

        foreach ($agencies as $agency) {
            // Check if latitude and longitude are missing
            if (empty($agency->latitude) || empty($agency->longitude)) {
                // Geocode the agency address if lat/lon not available
                $location = $this->geocodeAddress($agency->address);
                if ($location) {
                    // Update the database with the fetched lat/lon
                    DB::table('agencies')
                        ->where('id', $agency->id)
                        ->update(['latitude' => $location['lat'], 'longitude' => $location['lng']]);
                    
                    $agency->latitude = $location['lat'];
                    $agency->longitude = $location['lng'];
                } else {
                    continue; // Skip this agency if geocoding fails
                }
            }

            // Calculate distance between user and agency
            $distance = $this->haversineGreatCircleDistance($userLat, $userLng, $agency->latitude, $agency->longitude);
            
            // Check if the distance is within 40 km
            if ($distance <= 40) {
                $nearbyAgencies[] = $agency;
            }
        }

        return view('directing-master.listing', compact('nearbyAgencies', 'user_address', 'agencies'));
    }
}
